@@ -4,7 +4,7 @@ macro_rules! from_err {
     ($from:ty, $to:tt, $var:tt) => {
         impl From<$from> for $to {
             fn from(e: $from) -> $to {
-                $to::$var(e)
+                $to::$var(e.into())
             }
         }
     };
