@@ -7,14 +7,12 @@ use crate::Request;
 use crate::{Error, ErrorKind};
 use bytes::Bytes;
 use http::{HeaderMap, HeaderName, StatusCode};
-use rand::distributions::Alphanumeric;
-use rand::{thread_rng, Rng};
-use std::convert::Into;
+use rand::distr::Alphanumeric;
+use rand::Rng;
 use std::fmt;
 use std::io;
 use std::ops::Drop;
 use std::path::Path;
-use std::string::ToString;
 use std::sync::Arc;
 use std::sync::RwLock;
 
@@ -153,7 +151,7 @@ impl Mock {
         assert_on_drop: bool,
     ) -> Mock {
         let inner = InnerMock {
-            id: thread_rng()
+            id: rand::rng()
                 .sample_iter(&Alphanumeric)
                 .map(char::from)
                 .take(24)
