@@ -160,6 +160,7 @@ impl Series {
             DataType::Boolean => any_values_to_bool(values, strict)?.into_series(),
             DataType::String => any_values_to_string(values, strict)?.into_series(),
             DataType::Binary => any_values_to_binary(values, strict)?.into_series(),
+            DataType::BinaryOffset => any_values_to_binary_offset(values, strict)?.into_series(),
             #[cfg(feature = "dtype-date")]
             DataType::Date => any_values_to_date(values, strict)?.into_series(),
             #[cfg(feature = "dtype-time")]
@@ -369,6 +370,31 @@ fn any_values_to_binary(values: &[AnyValue], strict: bool) -> PolarsResult<Binar
     }
 }
 
+fn any_values_to_binary_offset(
+    values: &[AnyValue],
+    strict: bool,
+) -> PolarsResult<BinaryOffsetChunked> {
+    let mut builder = MutableBinaryArray::<i64>::new();
+    for av in values {
+        match av {
+            AnyValue::Binary(s) => builder.push(Some(*s)),
+            AnyValue::BinaryOwned(s) => builder.push(Some(&**s)),
+            AnyValue::Null => builder.push_null(),
+            av => {
+                if strict {
+                    return Err(invalid_value_error(&DataType::Binary, av));
+                } else {
+                    builder.push_null();
+                };
+            },
+        }
+    }
+    Ok(BinaryOffsetChunked::with_chunk(
+        Default::default(),
+        builder.into(),
+    ))
+}
+
 #[cfg(feature = "dtype-date")]
 fn any_values_to_date(values: &[AnyValue], strict: bool) -> PolarsResult<DateChunked> {
     let mut builder = PrimitiveChunkedBuilder::<Int32Type>::new(PlSmallStr::EMPTY, values.len());
@@ -387,7 +413,7 @@ fn any_values_to_date(values: &[AnyValue], strict: bool) -> PolarsResult<DateChu
             },
         }
     }
-    Ok(builder.finish().into())
+    Ok(builder.finish().into_date())
 }
 
 #[cfg(feature = "dtype-time")]
@@ -408,7 +434,7 @@ fn any_values_to_time(values: &[AnyValue], strict: bool) -> PolarsResult<TimeChu
             },
         }
     }
-    Ok(builder.finish().into())
+    Ok(builder.finish().into_time())
 }
 
 #[cfg(feature = "dtype-datetime")]
