@@ -3,6 +3,6 @@
 use std::sync::Arc;
 
 pub use polars_parquet::parquet::metadata::FileMetadata;
-pub use polars_parquet::read::statistics::{deserialize, Statistics as ParquetStatistics};
+pub use polars_parquet::read::statistics::{Statistics as ParquetStatistics, deserialize};
 
 pub type FileMetadataRef = Arc<FileMetadata>;
