@@ -1,8 +1,10 @@
 use polars_io::RowIndex;
 #[cfg(feature = "is_between")]
 use polars_ops::prelude::ClosedInterval;
+use polars_utils::slice_enum::Slice;
 
 use super::*;
+use crate::dsl;
 
 #[test]
 #[cfg(feature = "parquet")]
@@ -115,7 +117,7 @@ fn test_parquet_statistics_no_skip() {
 fn test_parquet_statistics() -> PolarsResult<()> {
     let _guard = SINGLE_LOCK.lock().unwrap();
     init_files();
-    std::env::set_var("POLARS_PANIC_IF_PARQUET_PARSED", "1");
+    unsafe { std::env::set_var("POLARS_PANIC_IF_PARQUET_PARSED", "1") };
     let par = true;
 
     // Test single predicates
@@ -136,7 +138,7 @@ fn test_parquet_statistics() -> PolarsResult<()> {
 
     // issue: 13427
     let out = scan_foods_parquet(par)
-        .filter(col("calories").is_in(lit(Series::new("".into(), [0, 500]))))
+        .filter(col("calories").is_in(lit(Series::new("".into(), [0, 500])), false))
         .collect()?;
     assert_eq!(out.shape(), (0, 4));
 
@@ -350,7 +352,7 @@ fn test_parquet_statistics() -> PolarsResult<()> {
         .collect()?;
     assert_eq!(out.shape(), (0, 4));
 
-    std::env::remove_var("POLARS_PANIC_IF_PARQUET_PARSED");
+    unsafe { std::env::remove_var("POLARS_PANIC_IF_PARQUET_PARSED") };
 
     Ok(())
 }
@@ -399,7 +401,9 @@ fn test_scan_parquet_limit_9001() {
             let sliced = options.slice.unwrap();
             sliced.1 == 3
         },
-        IR::Scan { file_options, .. } => file_options.slice == Some((0, 3)),
+        IR::Scan {
+            unified_scan_args, ..
+        } => unified_scan_args.pre_slice == Some(Slice::Positive { offset: 0, len: 3 }),
         _ => true,
     });
 }
@@ -504,9 +508,41 @@ fn test_union_and_agg_projections() -> PolarsResult<()> {
     let _guard = SINGLE_LOCK.lock().unwrap();
     // a union vstacks columns and aggscan optimization determines columns to aggregate in a
     // hashmap, if that doesn't set them sorted the vstack will panic.
-    let lf1 = LazyFrame::scan_parquet(GLOB_PARQUET, Default::default())?;
-    let lf2 = LazyFrame::scan_ipc(GLOB_IPC, Default::default())?;
-    let lf3 = LazyCsvReader::new(GLOB_CSV).finish()?;
+    let lf1: LazyFrame = DslBuilder::scan_parquet(
+        ScanSources::Paths([GLOB_PARQUET.into()].into()),
+        Default::default(),
+        UnifiedScanArgs {
+            extra_columns_policy: ExtraColumnsPolicy::Ignore,
+            ..Default::default()
+        },
+    )
+    .unwrap()
+    .build()
+    .into();
+
+    let lf2: LazyFrame = DslBuilder::scan_ipc(
+        ScanSources::Paths([GLOB_IPC.into()].into()),
+        Default::default(),
+        UnifiedScanArgs {
+            extra_columns_policy: ExtraColumnsPolicy::Ignore,
+            ..Default::default()
+        },
+    )
+    .unwrap()
+    .build()
+    .into();
+
+    let lf3: LazyFrame = DslBuilder::scan_csv(
+        ScanSources::Paths([GLOB_CSV.into()].into()),
+        Default::default(),
+        UnifiedScanArgs {
+            extra_columns_policy: ExtraColumnsPolicy::Ignore,
+            ..Default::default()
+        },
+    )
+    .unwrap()
+    .build()
+    .into();
 
     for lf in [lf1, lf2, lf3] {
         let lf = lf.filter(col("category").eq(lit("vegetables"))).select([
@@ -687,6 +723,54 @@ fn scan_anonymous_fn_with_options() -> PolarsResult<()> {
     Ok(())
 }
 
+#[test]
+fn scan_anonymous_fn_count() -> PolarsResult<()> {
+    struct MyScan {}
+
+    impl AnonymousScan for MyScan {
+        fn as_any(&self) -> &dyn std::any::Any {
+            self
+        }
+
+        fn allows_projection_pushdown(&self) -> bool {
+            true
+        }
+
+        fn scan(&self, scan_opts: AnonymousScanArgs) -> PolarsResult<DataFrame> {
+            assert_eq!(scan_opts.with_columns.as_deref(), Some(&["A".into()][..]));
+
+            Ok(fruits_cars()
+                .select(scan_opts.with_columns.unwrap().iter().cloned())
+                .unwrap())
+        }
+    }
+
+    let function = Arc::new(MyScan {});
+
+    let args = ScanArgsAnonymous {
+        schema: Some(fruits_cars().schema().clone()),
+        ..ScanArgsAnonymous::default()
+    };
+
+    let df = LazyFrame::anonymous_scan(function, args)?
+        .select(&[dsl::len()])
+        .collect()
+        .unwrap();
+
+    assert_eq!(df.get_columns().len(), 1);
+    assert_eq!(df.get_columns()[0].len(), 1);
+    assert_eq!(
+        df.get_columns()[0]
+            .cast(&DataType::UInt32)
+            .unwrap()
+            .as_materialized_series()
+            .first(),
+        Scalar::new(DataType::UInt32, AnyValue::UInt32(5))
+    );
+
+    Ok(())
+}
+
 #[test]
 #[cfg(feature = "dtype-full")]
 fn scan_small_dtypes() -> PolarsResult<()> {
