@@ -6,17 +6,14 @@ pub(crate) fn row_index_at_scan(q: LazyFrame) -> bool {
     let lp = q.optimize(&mut lp_arena, &mut expr_arena).unwrap();
 
     (&lp_arena).iter(lp).any(|(_, lp)| {
-        use IR::*;
-        matches!(
-            lp,
-            Scan {
-                file_options: FileScanOptions {
-                    row_index: Some(_),
-                    ..
-                },
-                ..
-            }
-        )
+        if let IR::Scan {
+            unified_scan_args, ..
+        } = lp
+        {
+            unified_scan_args.row_index.is_some()
+        } else {
+            false
+        }
     })
 }
 
@@ -50,34 +47,6 @@ pub(crate) fn predicate_at_all_scans(q: LazyFrame) -> bool {
     })
 }
 
-#[cfg(feature = "streaming")]
-pub(crate) fn is_pipeline(q: LazyFrame) -> bool {
-    let (mut expr_arena, mut lp_arena) = get_arenas();
-    let lp = q.optimize(&mut lp_arena, &mut expr_arena).unwrap();
-    matches!(
-        lp_arena.get(lp),
-        IR::MapFunction {
-            function: FunctionIR::Pipeline { .. },
-            ..
-        }
-    )
-}
-
-#[cfg(feature = "streaming")]
-pub(crate) fn has_pipeline(q: LazyFrame) -> bool {
-    let (mut expr_arena, mut lp_arena) = get_arenas();
-    let lp = q.optimize(&mut lp_arena, &mut expr_arena).unwrap();
-    (&lp_arena).iter(lp).any(|(_, lp)| {
-        matches!(
-            lp,
-            IR::MapFunction {
-                function: FunctionIR::Pipeline { .. },
-                ..
-            }
-        )
-    })
-}
-
 #[cfg(any(feature = "parquet", feature = "csv"))]
 fn slice_at_scan(q: LazyFrame) -> bool {
     let (mut expr_arena, mut lp_arena) = get_arenas();
@@ -85,7 +54,9 @@ fn slice_at_scan(q: LazyFrame) -> bool {
     (&lp_arena).iter(lp).any(|(_, lp)| {
         use IR::*;
         match lp {
-            Scan { file_options, .. } => file_options.slice.is_some(),
+            Scan {
+                unified_scan_args, ..
+            } => unified_scan_args.pre_slice.is_some(),
             _ => false,
         }
     })
