@@ -11,7 +11,6 @@ impl AExpr {
 
         match self {
             Column(_) | Literal(_) | Len => {},
-            Alias(e, _) => container.extend([*e]),
             BinaryExpr { left, op: _, right } => {
                 container.extend([*right, *left]);
             },
@@ -55,6 +54,15 @@ impl AExpr {
                 container.extend(partition_by.iter().rev().cloned());
                 container.extend([*function]);
             },
+            Eval {
+                expr,
+                evaluation,
+                variant: _,
+            } => {
+                // We don't use the evaluation here because it does not contain inputs.
+                _ = evaluation;
+                container.extend([*expr]);
+            },
             Slice {
                 input,
                 offset,
@@ -69,7 +77,6 @@ impl AExpr {
         use AExpr::*;
         let input = match &mut self {
             Column(_) | Literal(_) | Len => return self,
-            Alias(input, _) => input,
             Cast { expr, .. } => expr,
             Explode { expr, .. } => expr,
             BinaryExpr { left, right, .. } => {
@@ -123,6 +130,15 @@ impl AExpr {
                 }
                 return self;
             },
+            Eval {
+                expr,
+                evaluation,
+                variant: _,
+            } => {
+                *expr = inputs[0];
+                _ = evaluation; // Intentional.
+                return self;
+            },
             Slice {
                 input,
                 offset,
