@@ -5,7 +5,8 @@ use super::*;
 
 /// Applies expression simplification and type coercion during conversion to IR.
 pub struct ConversionOptimizer {
-    scratch: Vec<Node>,
+    scratch: Vec<(Node, usize)>,
+    schemas: Vec<Schema>,
 
     simplify: Option<SimplifyExprRule>,
     coerce: Option<TypeCoercionRule>,
@@ -18,6 +19,17 @@ pub struct ConversionOptimizer {
     pub(super) used_arenas: PlHashSet<u32>,
 }
 
+struct ExtendVec<'a> {
+    out: &'a mut Vec<(Node, usize)>,
+    schema_idx: usize,
+}
+impl Extend<Node> for ExtendVec<'_> {
+    fn extend<T: IntoIterator<Item = Node>>(&mut self, iter: T) {
+        self.out
+            .extend(iter.into_iter().map(|n| (n, self.schema_idx)))
+    }
+}
+
 impl ConversionOptimizer {
     pub fn new(simplify: bool, type_coercion: bool, type_check: bool) -> Self {
         let simplify = if simplify {
@@ -40,6 +52,7 @@ impl ConversionOptimizer {
 
         ConversionOptimizer {
             scratch: Vec::with_capacity(8),
+            schemas: Vec::new(),
             simplify,
             coerce,
             check,
@@ -48,10 +61,13 @@ impl ConversionOptimizer {
     }
 
     pub fn push_scratch(&mut self, expr: Node, expr_arena: &Arena<AExpr>) {
-        self.scratch.push(expr);
+        self.scratch.push((expr, 0));
         // traverse all subexpressions and add to the stack
         let expr = unsafe { expr_arena.get_unchecked(expr) };
-        expr.inputs_rev(&mut self.scratch);
+        expr.inputs_rev(&mut ExtendVec {
+            out: &mut self.scratch,
+            schema_idx: 0,
+        });
     }
 
     pub fn fill_scratch<N: Borrow<Node>>(&mut self, exprs: &[N], expr_arena: &Arena<AExpr>) {
@@ -78,23 +94,65 @@ impl ConversionOptimizer {
         }
 
         // process the expressions on the stack and apply optimizations.
-        while let Some(current_expr_node) = self.scratch.pop() {
+        let schema = get_schema(ir_arena, current_ir_node);
+        let plan = ir_arena.get(current_ir_node);
+        let mut ctx = OptimizeExprContext::default();
+        #[cfg(feature = "python")]
+        {
+            use crate::dsl::python_dsl::PythonScanSource;
+            ctx.in_pyarrow_scan = matches!(plan, IR::PythonScan { options } if options.python_source == PythonScanSource::Pyarrow);
+            ctx.in_io_plugin = matches!(plan, IR::PythonScan { options } if options.python_source == PythonScanSource::IOPlugin);
+        };
+        ctx.in_filter = matches!(plan, IR::Filter { .. });
+        ctx.has_inputs = !get_input(ir_arena, current_ir_node).is_empty();
+
+        self.schemas.clear();
+        while let Some((current_expr_node, schema_idx)) = self.scratch.pop() {
             let expr = unsafe { expr_arena.get_unchecked(current_expr_node) };
 
             if expr.is_leaf() {
                 continue;
             }
 
+            // Evaluation expressions still need to do rules on the evaluation expression but the
+            // schema is not the same and it is not concluded in the inputs. Therefore, we handl
+            if let AExpr::Eval {
+                expr,
+                evaluation,
+                variant,
+            } = expr
+            {
+                let schema = if schema_idx == 0 {
+                    &schema
+                } else {
+                    &self.schemas[schema_idx - 1]
+                };
+                let expr = expr_arena
+                    .get(*expr)
+                    .get_type(schema, Context::Default, expr_arena)?;
+
+                let element_dtype = variant.element_dtype(&expr)?;
+                let schema = Schema::from_iter([(PlSmallStr::EMPTY, element_dtype.clone())]);
+                self.schemas.push(schema);
+                self.scratch.push((*evaluation, self.schemas.len()));
+            }
+
+            let schema = if schema_idx == 0 {
+                &schema
+            } else {
+                &self.schemas[schema_idx - 1]
+            };
+
             if let Some(rule) = &mut self.simplify {
                 while let Some(x) =
-                    rule.optimize_expr(expr_arena, current_expr_node, ir_arena, current_ir_node)?
+                    rule.optimize_expr(expr_arena, current_expr_node, schema, ctx)?
                 {
                     expr_arena.replace(current_expr_node, x);
                 }
             }
             if let Some(rule) = &mut self.coerce {
                 while let Some(x) =
-                    rule.optimize_expr(expr_arena, current_expr_node, ir_arena, current_ir_node)?
+                    rule.optimize_expr(expr_arena, current_expr_node, schema, ctx)?
                 {
                     expr_arena.replace(current_expr_node, x);
                 }
@@ -102,7 +160,10 @@ impl ConversionOptimizer {
 
             let expr = unsafe { expr_arena.get_unchecked(current_expr_node) };
             // traverse subexpressions and add to the stack
-            expr.inputs_rev(&mut self.scratch)
+            expr.inputs_rev(&mut ExtendVec {
+                out: &mut self.scratch,
+                schema_idx,
+            });
         }
 
         Ok(())
