@@ -5,15 +5,22 @@ use std::sync::{Arc, LazyLock, RwLock};
 use pyo3::{Py, PyAny, PyResult};
 
 pub type PythonToSinkTarget = Arc<dyn Fn(Py<PyAny>) -> PyResult<Box<dyn Any>> + Send + Sync>;
+pub type DataFrameToPython = Arc<dyn Fn(Box<dyn Any>) -> PyResult<Py<PyAny>> + Send + Sync>;
 
 #[derive(Clone)]
 pub struct FromPythonConvertRegistry {
     pub sink_target: PythonToSinkTarget,
 }
 
+#[derive(Clone)]
+pub struct ToPythonConvertRegistry {
+    pub df: DataFrameToPython,
+}
+
 #[derive(Clone)]
 pub struct PythonConvertRegistry {
     pub from_py: FromPythonConvertRegistry,
+    pub to_py: ToPythonConvertRegistry,
 }
 
 static PYTHON_CONVERT_REGISTRY: LazyLock<RwLock<Option<PythonConvertRegistry>>> =
