@@ -144,6 +144,9 @@ pub struct IdeMenu {
     values: Vec<Suggestion>,
     /// Selected value. Starts at 0
     selected: u16,
+    /// Number of values that are skipped when printing,
+    /// depending on selected value and terminal height
+    skip_values: u16,
     /// Event sent to the menu
     event: Option<MenuEvent>,
     /// Longest suggestion found in the values
@@ -161,6 +164,7 @@ impl Default for IdeMenu {
             working_details: IdeMenuDetails::default(),
             values: Vec::new(),
             selected: 0,
+            skip_values: 0,
             event: None,
             longest_suggestion: 0,
             input: None,
@@ -447,7 +451,7 @@ impl IdeMenu {
                         RESET
                     );
                 } else {
-                    *line = format!("{}{}", line, padding);
+                    *line = format!("{line}{padding}");
                 }
             }
         }
@@ -518,18 +522,24 @@ impl IdeMenu {
             let shortest_base = shortest_base
                 .strip_prefix(is_quote)
                 .unwrap_or(shortest_base);
-            let match_len = shortest_base.len().min(string.len());
+            let match_len = shortest_base.chars().count().min(string.chars().count());
 
             // Find match position - look for the base string in the suggestion (case-insensitive)
-            let match_position = suggestion
-                .value
+            let match_position = string
                 .to_lowercase()
                 .find(&shortest_base.to_lowercase())
                 .unwrap_or(0);
 
             // The match is just the part that matches the shortest_base
-            let match_str = &string
-                [match_position..match_position + match_len.min(string.len() - match_position)];
+            let match_str = {
+                let match_str = &string[match_position..];
+                let match_len_bytes = match_str
+                    .char_indices()
+                    .nth(match_len)
+                    .map(|(i, _)| i)
+                    .unwrap_or_else(|| match_str.len());
+                &string[match_position..match_position + match_len_bytes]
+            };
 
             // Prefix is everything before the match
             let prefix = &string[..match_position];
@@ -826,6 +836,29 @@ impl Menu for IdeMenu {
 
             self.working_details.space_left = space_left;
             self.working_details.space_right = space_right;
+
+            let mut available_lines = painter
+                .remaining_lines_real()
+                .min(self.default_details.max_completion_height);
+
+            // Handle the case where a prompt uses the entire screen.
+            // Drawing the menu has priority over the drawing the prompt.
+            if available_lines == 0 {
+                available_lines = painter.remaining_lines().min(self.min_rows());
+            }
+
+            let visible_items = available_lines.saturating_sub(border_width);
+
+            self.skip_values = if self.selected <= self.skip_values {
+                // Selection is above the visible area
+                self.selected
+            } else if self.selected >= self.skip_values + visible_items {
+                // Selection is below the visible area
+                self.selected.saturating_sub(visible_items) + 1
+            } else {
+                // Selection is within the visible area
+                self.skip_values
+            }
         }
     }
 
@@ -859,17 +892,7 @@ impl Menu for IdeMenu {
             };
 
             let available_lines = available_lines.min(self.default_details.max_completion_height);
-            // The skip values represent the number of lines that should be skipped
-            // while printing the menu
-            let skip_values = if self.selected >= available_lines.saturating_sub(border_width) {
-                let skip_lines = self
-                    .selected
-                    .saturating_sub(available_lines.saturating_sub(border_width))
-                    + 1;
-                skip_lines as usize
-            } else {
-                0
-            };
+            let skip_values = self.skip_values as usize;
 
             let available_values = available_lines.saturating_sub(border_width) as usize;
 
@@ -957,7 +980,7 @@ impl Menu for IdeMenu {
                             )
                         }
                         Left(suggestion_line) => {
-                            strings[idx] = format!("{}{}", distance_left, suggestion_line);
+                            strings[idx] = format!("{distance_left}{suggestion_line}");
                         }
                         Right(description_line) => strings.push(format!(
                             "{}{}",
@@ -999,7 +1022,7 @@ impl Menu for IdeMenu {
                             );
                         }
                         Right(description_line) => {
-                            strings.push(format!("{}{}", distance_left, description_line,))
+                            strings.push(format!("{distance_left}{description_line}",))
                         }
                     }
                 }
@@ -1487,4 +1510,30 @@ mod tests {
         menu.update_values(&mut editor, &mut completer);
         assert!(menu.menu_string(2, true).contains("おは"));
     }
+
+    #[test]
+    fn test_menu_create_value_string_starting_with_multibyte_char() {
+        // https://github.com/nushell/nushell/issues/15938
+        let mut completer = FakeCompleter::new(&["验abc/"]);
+        let mut menu = IdeMenu::default().with_name("testmenu");
+        menu.working_details.completion_width = 50;
+        let mut editor = Editor::default();
+
+        editor.set_buffer("ac".to_string(), UndoBehavior::CreateUndoPoint);
+        menu.update_values(&mut editor, &mut completer);
+        assert!(menu.menu_string(10, true).contains("验"));
+    }
+
+    #[test]
+    fn test_menu_create_value_string_long_unicode_string() {
+        // Test for possible panic if a long filename gets truncated
+        let mut completer = FakeCompleter::new(&[&("验".repeat(205) + "abc/")]);
+        let mut menu = IdeMenu::default().with_name("testmenu");
+        menu.working_details.completion_width = 50;
+        let mut editor = Editor::default();
+
+        editor.set_buffer("a".to_string(), UndoBehavior::CreateUndoPoint);
+        menu.update_values(&mut editor, &mut completer);
+        assert!(menu.menu_string(10, true).contains("验"));
+    }
 }
