@@ -1,6 +1,41 @@
-//! This module contains a list of primitives that implement a [`Object`] trait.
+//! This module contains a list of primitives that implement a [`Object`] trait.\
 //! They help to locate a necessary segment on a [`Table`].
 //!
+//! # Example
+//!
+//! ```
+//! use tabled::{
+//!     settings::{
+//!         object::{Columns, Object, Rows},
+//!         Alignment,
+//!     },
+//!     assert::assert_table,
+//!     Table,
+//! };
+//!
+//! let data = [
+//!     [1, 2, 3, 4, 5],
+//!     [10, 20, 30, 40, 50],
+//!     [100, 200, 300, 400, 500],
+//! ];
+//!
+//! let mut table = Table::new(data);
+//! table.modify(Rows::first().not(Columns::first()), Alignment::right());
+//!
+//! assert_table!(
+//!     table,
+//!     "+-----+-----+-----+-----+-----+"
+//!     "| 0   |   1 |   2 |   3 |   4 |"
+//!     "+-----+-----+-----+-----+-----+"
+//!     "| 1   | 2   | 3   | 4   | 5   |"
+//!     "+-----+-----+-----+-----+-----+"
+//!     "| 10  | 20  | 30  | 40  | 50  |"
+//!     "+-----+-----+-----+-----+-----+"
+//!     "| 100 | 200 | 300 | 400 | 500 |"
+//!     "+-----+-----+-----+-----+-----+"
+//! );
+//! ```
+//!
 //! [`Table`]: crate::Table
 
 mod cell;
@@ -28,6 +63,7 @@ pub use iterator::{
     FilterObject, FilterObjectIter, ObjectIterator, SkipObject, SkipObjectIter, StepByObject,
     StepByObjectIter,
 };
+use papergrid::config::Position;
 pub use rows::{FirstRow, LastRow, LastRowOffset, Row, Rows, RowsIter};
 pub use segment::{SectorIter, Segment, SegmentAll};
 
@@ -224,7 +260,7 @@ where
 pub struct UnionIter<L, R> {
     lhs: Option<L>,
     rhs: R,
-    seen: HashSet<(usize, usize)>,
+    seen: HashSet<Position>,
     current: Option<EntityIterator>,
     count_rows: usize,
     count_cols: usize,
@@ -267,7 +303,8 @@ where
                 }
 
                 let _ = self.seen.insert(p);
-                return Some(Entity::Cell(p.0, p.1));
+
+                return Some(p.into());
             }
         }
 
@@ -277,7 +314,8 @@ where
                 if let Some(p) = iter.by_ref().next() {
                     let _ = self.seen.insert(p);
                     self.current = Some(iter);
-                    return Some(Entity::Cell(p.0, p.1));
+
+                    return Some(p.into());
                 }
             }
 
@@ -291,7 +329,8 @@ where
                 if !self.seen.contains(&p) {
                     let _ = self.seen.insert(p);
                     self.current = Some(iter);
-                    return Some(Entity::Cell(p.0, p.1));
+
+                    return Some(p.into());
                 }
             }
         }
@@ -304,7 +343,7 @@ where
 #[derive(Debug)]
 pub struct DiffIter<L> {
     lhs: L,
-    seen: HashSet<(usize, usize)>,
+    seen: HashSet<Position>,
     count_rows: usize,
     count_cols: usize,
     current: Option<EntityIterator>,
@@ -348,7 +387,7 @@ where
         if let Some(iter) = self.current.as_mut() {
             for p in iter.by_ref() {
                 if !self.seen.contains(&p) {
-                    return Some(Entity::Cell(p.0, p.1));
+                    return Some(p.into());
                 }
             }
         }
@@ -359,7 +398,7 @@ where
             for p in iter.by_ref() {
                 if !self.seen.contains(&p) {
                     self.current = Some(iter);
-                    return Some(Entity::Cell(p.0, p.1));
+                    return Some(p.into());
                 }
             }
         }
@@ -372,7 +411,7 @@ where
 #[derive(Debug)]
 pub struct IntersectIter<L> {
     lhs: L,
-    seen: HashSet<(usize, usize)>,
+    seen: HashSet<Position>,
     count_rows: usize,
     count_cols: usize,
     current: Option<EntityIterator>,
@@ -416,7 +455,7 @@ where
         if let Some(iter) = self.current.as_mut() {
             for p in iter.by_ref() {
                 if self.seen.contains(&p) {
-                    return Some(Entity::Cell(p.0, p.1));
+                    return Some(p.into());
                 }
             }
         }
@@ -427,7 +466,7 @@ where
             for p in iter.by_ref() {
                 if self.seen.contains(&p) {
                     self.current = Some(iter);
-                    return Some(Entity::Cell(p.0, p.1));
+                    return Some(p.into());
                 }
             }
         }
@@ -440,7 +479,7 @@ where
 #[derive(Debug)]
 pub struct InversionIter {
     all: SectorCellsIter,
-    seen: HashSet<(usize, usize)>,
+    seen: HashSet<Position>,
 }
 
 impl InversionIter {
@@ -470,7 +509,7 @@ impl Iterator for InversionIter {
     fn next(&mut self) -> Option<Self::Item> {
         for p in self.all.by_ref() {
             if !self.seen.contains(&p) {
-                return Some(Entity::Cell(p.0, p.1));
+                return Some(p.into());
             }
         }
 
@@ -705,11 +744,11 @@ mod tests {
             [Entity::Cell(0, 1), Entity::Cell(0, 2)]
         );
         assert_eq!(
-            vec_cells(Columns::single(1).not(Rows::single(1)), 3, 3),
+            vec_cells(Columns::one(1).not(Rows::one(1)), 3, 3),
             [Entity::Cell(0, 1), Entity::Cell(2, 1)]
         );
         assert_eq!(
-            vec_cells(Rows::single(1).not(Columns::single(1)), 3, 3),
+            vec_cells(Rows::one(1).not(Columns::one(1)), 3, 3),
             [Entity::Cell(1, 0), Entity::Cell(1, 2)]
         );
     }
@@ -721,7 +760,7 @@ mod tests {
             []
         );
         assert_eq!(
-            vec_cells(Segment::all().intersect(Rows::single(1)), 2, 3),
+            vec_cells(Segment::all().intersect(Rows::one(1)), 2, 3),
             [Entity::Cell(1, 0), Entity::Cell(1, 1), Entity::Cell(1, 2)]
         );
         assert_eq!(
@@ -733,10 +772,7 @@ mod tests {
             [Entity::Cell(0, 0)]
         );
         // maybe we somehow shall not limit the rows/columns by the max count?
-        assert_eq!(
-            vec_cells(Rows::single(1).intersect(Columns::single(1)), 2, 1),
-            []
-        );
+        assert_eq!(vec_cells(Rows::one(1).intersect(Columns::one(1)), 2, 1), []);
     }
 
     #[test]
